package com.jcodecraeer.xrecyclerview;

import android.content.Context;
import android.content.res.Resources;
import android.support.v7.widget.RecyclerView;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

public class LoadingMoreFooter extends LinearLayout {

    private SimpleViewSwitcher progressCon;
    //to prevent leave space when state is STATE_COMPLETE
    private LinearLayout mContain;
    public final static int STATE_HIDE = -1;
    public final static int STATE_INIT = 0;
    public final static int STATE_LOADING = 1;
    public final static int STATE_COMPLETE = 2;
    public final static int STATE_NOMORE = 3;
    private TextView mText;
    private String loadingHint;
    private String noMoreHint;
    private String loadingDoneHint;
    public int mMeasuredHeight;

	public LoadingMoreFooter(Context context) {
		super(context);
		initView();
	}

	/**
	 * @param context
	 * @param attrs
	 */
	public LoadingMoreFooter(Context context, AttributeSet attrs) {
		super(context, attrs);
		initView();
	}

    public void setLoadingHint(String hint) {
        loadingHint = hint;
    }

    public void setNoMoreHint(String hint) {
        noMoreHint = hint;
    }

    public void setLoadingDoneHint(String hint) {
        loadingDoneHint = hint;
    }

    public void initView(){
        setGravity(Gravity.CENTER);
        setLayoutParams(new RecyclerView.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));

        //额外增加一个container的原因是RecyclerView会让View.GONE的view也占据高度，至于原因就不知道了...
        mContain = new LinearLayout(getContext());
        addView(mContain,new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT,LinearLayout.LayoutParams.WRAP_CONTENT));
        mContain.setGravity(Gravity.CENTER);
        mContain.setPadding(0,(int)convertDpToPixel(8,getContext()),0,(int)convertDpToPixel(8,getContext()));
        progressCon = new SimpleViewSwitcher(getContext());
        progressCon.setLayoutParams(new ViewGroup.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT));

        ProgressBar progressBar = new ProgressBar(getContext(), null,
                android.R.attr.progressBarStyle);
        LayoutParams params = new LayoutParams((int)convertDpToPixel(30,getContext()),
                (int)convertDpToPixel(30,getContext()));
        progressBar.setLayoutParams(params);
        progressCon.setView(progressBar);

        mContain.addView(progressCon);
        mText = new TextView(getContext());
        mText.setText("正在加载...");
        loadingHint = (String)getContext().getText(R.string.listview_loading);
        noMoreHint = (String)getContext().getText(R.string.nomore_loading);
        loadingDoneHint = (String)getContext().getText(R.string.loading_done);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        layoutParams.setMargins( (int)getResources().getDimension(R.dimen.textandiconmargin),0,0,0 );

        mText.setLayoutParams(layoutParams);
        mContain.addView(mText);
        setState(STATE_INIT);
    }

    public void setProgressView(View view){
        progressCon.setView(view);
    }

    public void  setState(int state) {
        switch(state) {
            case STATE_INIT:
                this.setVisibility(View.GONE);
                break;
            case STATE_HIDE:
                this.setVisibility(View.GONE);
                mContain .setVisibility(View.GONE);
                break;
            case STATE_LOADING:
                progressCon.setVisibility(View.VISIBLE);
                mText.setText(loadingHint);
                this.setVisibility(View.VISIBLE);
                break;
            case STATE_COMPLETE:
                mText.setText(loadingDoneHint);
                progressCon.setVisibility(View.INVISIBLE);//需要占据原来的空间，不然下拉加载显示不全
                this.setVisibility(View.VISIBLE);
                break;
            case STATE_NOMORE:
                mText.setText(noMoreHint);
                progressCon.setVisibility(View.GONE);
                this.setVisibility(View.VISIBLE);
                break;
        }
    }

    public static float convertDpToPixel(float dp, Context context) {
        return dp * (getDisplayMetrics(context).densityDpi / 160f);
    }

    public static DisplayMetrics getDisplayMetrics(Context context) {
        Resources resources = context.getResources();
        return resources.getDisplayMetrics();
    }
//    @Override
//    public void setVisibility(int visibility) {
//        super.setVisibility(visibility);
//        mContain .setVisibility(visibility);
//        Log.i("sss",visibility+"sss");
//    }
}
